'''
Created on Oct 19, 2015
@author: hari
'''
from __future__ import print_function
from builtins import range
from pycoils.trees.splaytree import SplayTree
import unittest

class Splay_Tree_Test_Empty_Tree(unittest.TestCase):    
    def setUp(self):
        self._splaytree = SplayTree()
    
    def test_node_count_of_empty_tree(self):
        self.assertEqual(self._splaytree.node_count, 0, 'Empty tree node count must be 0')
        
    def test_find_key_empty_tree(self):
        self.assertEqual(self._splaytree.find(20), None, 'Empty tree find operation must return None')
    
    def test_delete_element_empty_tree(self):
        self._splaytree.remove(20)
    
    def tearDown(self):
        self._splaytree = None

class Splay_Tree_Test_Tree_With_1_Element(unittest.TestCase):
    
    def setUp(self):
        self._splaytree = SplayTree()
        self._splaytree.insert(key = 10, obj = 10)
    
    def test_node_count_of_tree_with_1_node(self):
        self.assertEqual(self._splaytree.node_count, 1, 'Tree node count must be 1')
        
    def test_find_key_of_tree_with_1_node(self):
        self.assertEqual(self._splaytree.find(10), 10, 'Find operation failed on tree with 1 node')
    
    def test_delete_element_of_tree_with_1_node(self):
        self._splaytree.remove(10)
        self.assertEqual(self._splaytree.node_count, 0, 'Empty tree node count must be 0')
        self.assertEqual(self._splaytree.find(20), None, 'Empty tree find operation must return None')
            
    def tearDown(self):
        self._splaytree = None

class Splay_Tree_Test_Tree_Count_Find_Element_With_10_Elements(unittest.TestCase):

    def setUp(self):
        self._splaytree = SplayTree()
        self._splaytree.insert(key = 5, obj = 5)
        self._splaytree.insert(key = 8, obj = 8)
        self._splaytree.insert(key = 7, obj = 7)
        self._splaytree.insert(key = 9, obj = 9)
        self._splaytree.insert(key = 10, obj = 10)
        self._splaytree.insert(key = 2, obj = 2)
        self._splaytree.insert(key = 1, obj = 1)
        self._splaytree.insert(key = 3, obj = 3)
        self._splaytree.insert(key = 4, obj = 4)
        self._splaytree.insert(key = 6, obj = 6)
        
        self._splay_tree_node_count = 10
    
    def test_node_count_of_tree_with_10_node(self):
        self.assertEqual(self._splaytree.node_count, self._splay_tree_node_count, 'Tree node count must be 10')
        
    def test_find_element_of_tree_with_10_node(self):
        for i in range(1, 11): # 1 to 10
            tree_element = self._splaytree.find(i)
            self.assertNotEqual(tree_element, None, 'BST findElement did not return existing element')
            self.assertEqual(tree_element, i, 'Find operation failed on tree with 1 node')
    
    def tearDown(self):
        self._splaytree = None

class Splay_Tree_Test_Tree_Delete_Element_With_10_Elements(unittest.TestCase):
    
    def setUp(self):
        self._splaytree = SplayTree()
        self._splaytree.insert(key = 5, obj = 5)
        self._splaytree.insert(key = 8, obj = 8)
        self._splaytree.insert(key = 7, obj = 7)
        self._splaytree.insert(key = 9, obj = 9)
        self._splaytree.insert(key = 10, obj = 10)
        self._splaytree.insert(key = 2, obj = 2)
        self._splaytree.insert(key = 1, obj = 1)
        self._splaytree.insert(key = 3, obj = 3)
        self._splaytree.insert(key = 4, obj = 4)
        self._splaytree.insert(key = 6, obj = 6)
        
        self._splay_tree_node_count = 10
        
    def test_delete_element_of_tree_non_existing_element(self):
        self._splaytree.remove(11)
        self.assertEqual(self._splaytree.node_count, self._splay_tree_node_count, 'Tree node count must be 10')
    
    def test_delete_element_of_tree_with_10_node(self):
        elements_to_delete = [10, 1, 7, 3, 5, 8, 2, 6, 9]
        for element in elements_to_delete:
        
            self._splaytree.remove(element)
            self.assertEqual(self._splaytree.find(element), None, 'Element found in BST after deleting it!')
            self._splay_tree_node_count = self._splay_tree_node_count - 1
            self.assertEqual(self._splaytree.node_count, self._splay_tree_node_count, 'Tree node count must tally after deletion')
    
    def tearDown(self):
        self._splaytree = None

class Splay_Tree_Test_Tree_Delete_Element_With_10_Sorted_Elements(unittest.TestCase):
    
    def setUp(self):
        self._splaytree = SplayTree()
        self._splaytree.insert(key = 1, obj = 1)
        self._splaytree.insert(key = 2, obj = 2)
        self._splaytree.insert(key = 3, obj = 3)
        self._splaytree.insert(key = 4, obj = 4)
        self._splaytree.insert(key = 5, obj = 5)
        self._splaytree.insert(key = 6, obj = 6)
        self._splaytree.insert(key = 7, obj = 7)
        self._splaytree.insert(key = 8, obj = 8)
        self._splaytree.insert(key = 9, obj = 9)
        self._splaytree.insert(key = 10, obj = 10)
        
        self._splay_tree_node_count = 10
        
    def test_delete_element_of_tree_non_existing_element(self):
        self._splaytree.remove(11)
        self.assertEqual(self._splaytree.node_count, self._splay_tree_node_count, 'Tree node count must be 10')
    
    def test_delete_element_of_tree_with_10_node(self):
        elements_to_delete = [10, 1, 7, 3, 5, 8, 2, 6, 9, 4]
        for element in elements_to_delete: 
            self._splaytree.remove(element)
            self.assertEqual(self._splaytree.find(element), None, 'Element found in BST after deleting it!')
            self._splay_tree_node_count = self._splay_tree_node_count - 1
            self.assertEqual(self._splaytree.node_count, self._splay_tree_node_count, 'Tree node count must tally after deletion')
    
    def tearDown(self):
        self._splaytree = None

class Splay_Tree_Test_Tree_Delete_Element_With_10_Reverse_Sorted_Elements(unittest.TestCase):
    
    def setUp(self):
        self._splaytree = SplayTree()
        self._splaytree.insert(key = 10, obj = 10)
        self._splaytree.insert(key = 9, obj = 9)
        self._splaytree.insert(key = 8, obj = 8)
        self._splaytree.insert(key = 7, obj = 7)
        self._splaytree.insert(key = 6, obj = 6)
        self._splaytree.insert(key = 5, obj = 5)
        self._splaytree.insert(key = 4, obj = 4)
        self._splaytree.insert(key = 3, obj = 3)
        self._splaytree.insert(key = 2, obj = 2)
        self._splaytree.insert(key = 1, obj = 1)
        
        self._splay_tree_node_count = 10
        
    def test_delete_element_of_tree_non_existing_element(self):
        self._splaytree.remove(11)
        self.assertEqual(self._splaytree.node_count, self._splay_tree_node_count, 'Tree node count must be 10')
    
    def test_delete_element_of_tree_with_10_node(self):
        elements_to_delete = [10, 1, 7, 3, 5, 8, 2, 6, 9, 4]
        for element in elements_to_delete: 
            self._splaytree.remove(element)
            self.assertEqual(self._splaytree.find(element), None, 'Element found in BST after deleting it!')
            self._splay_tree_node_count = self._splay_tree_node_count - 1
            self.assertEqual(self._splaytree.node_count, self._splay_tree_node_count, 'Tree node count must tally after deletion')
            
    def tearDown(self):
        self._splaytree = None
        
class Splay_Tree_Test_Has_Key_with_Empty_Tree(unittest.TestCase):    
    def setUp(self):
        self._splaytree = SplayTree()
    
    def test_has_key_empty_tree(self):
        self.assertEqual(20 in self._splaytree, False, 'Empty tree has_key operation must return False')
    
    def tearDown(self):
        self._splaytree = None

class Splay_Tree_Test_Has_Key_with_Single_Node_Tree(unittest.TestCase):    
    def setUp(self):
        self._splaytree = SplayTree()
        self._splaytree.insert(key = 10, obj = 'Object value')
    
    def test_has_key_empty_tree(self):
        self.assertEqual(10 in self._splaytree, True, 'Tree has_key operation must return True for existing key')
    
    def tearDown(self):
        self._splaytree = None

class Splay_Tree_Test_Has_Key_With_10_Elements(unittest.TestCase):
    
    def setUp(self):
        self._splaytree = SplayTree()
        self._splaytree.insert(key = 5, obj = 5)
        self._splaytree.insert(key = 8, obj = 8)
        self._splaytree.insert(key = 7, obj = 7)
        self._splaytree.insert(key = 9, obj = 9)
        self._splaytree.insert(key = 10, obj = 10)
        self._splaytree.insert(key = 2, obj = 2)
        self._splaytree.insert(key = 1, obj = 1)
        self._splaytree.insert(key = 3, obj = 3)
        self._splaytree.insert(key = 4, obj = 4)
        self._splaytree.insert(key = 6, obj = 6)
        
        self._splay_tree_node_count = 10
    
    def test_has_key_with_10_node(self):
        for i in range(1, 11): # 1 to 10
            self.assertEqual(i in self._splaytree, True, 'has_key operation must return True for existing key.')
    
    def tearDown(self):
        self._splaytree = None
#
#Splay tree tests that do ops and check nodes with in-order traversal
#
class Splay_Tree_Test_Tree_Structure_After_Insert_With_Inorder_Traversal(unittest.TestCase):
    '''
    The purpose of this test is to make sure that the splay op does not ruin the tree. So after each insert we check the
    tree structure.
    '''
    def setUp(self):
        self._splaytree = SplayTree()
        
    def test_tree_structure_after_each_insert_with_inorder_traversal(self):
        expected_inorder_list = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
        inorder_from_tree = []
        for i in range(1, 11): # 1 to 10
            self._splaytree.insert(i, i)
            for key, element in self._splaytree.traversal(want_in_order = True):
                inorder_from_tree.append(element)
            self.assertEqual(inorder_from_tree, expected_inorder_list[:len(inorder_from_tree)], 'Tree ruined during insert splays')
            inorder_from_tree = []
        
    def tearDown(self):
        self._splaytree = None

class Splay_Tree_Test_Tree_Structure_After_Delete_With_Inorder_Traversal(unittest.TestCase):
    '''
    The purpose of this test is to make sure that the splay op does not ruin the tree. So after each insert we check the
    tree structure.
    '''
    def setUp(self):
        self._splaytree = SplayTree()
        self._splaytree.insert(key = 5, obj = 5)
        self._splaytree.insert(key = 8, obj = 8)
        self._splaytree.insert(key = 7, obj = 7)
        self._splaytree.insert(key = 9, obj = 9)
        self._splaytree.insert(key = 10, obj = 10)
        self._splaytree.insert(key = 2, obj = 2)
        self._splaytree.insert(key = 1, obj = 1)
        self._splaytree.insert(key = 3, obj = 3)
        self._splaytree.insert(key = 4, obj = 4)
        self._splaytree.insert(key = 6, obj = 6)

    def test_tree_structure_after_each_delete_with_inorder_traversal(self):
        expected_inorder_list = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
        inorder_from_tree = []
        for i in range(1, 11): # 1 to 10
            self._splaytree.remove(i)
            for key, element in self._splaytree.traversal(want_in_order = True):
                inorder_from_tree.append(element)
            self.assertEqual(inorder_from_tree, expected_inorder_list[i:10], 'Tree ruined during insert splays')
            inorder_from_tree = []
        
    def tearDown(self):
        self._splaytree = None


class Splay_Tree_Test_Tree_Structure_After_Find_With_Inorder_Traversal(unittest.TestCase):
    '''
    The purpose of this test is to make sure that the splay op does not ruin the tree. So after each insert we check the
    tree structure.
    '''
    def setUp(self):
        self._splaytree = SplayTree()
        self._splaytree.insert(key = 5, obj = 5)
        self._splaytree.insert(key = 8, obj = 8)
        self._splaytree.insert(key = 7, obj = 7)
        self._splaytree.insert(key = 9, obj = 9)
        self._splaytree.insert(key = 10, obj = 10)
        self._splaytree.insert(key = 2, obj = 2)
        self._splaytree.insert(key = 1, obj = 1)
        self._splaytree.insert(key = 3, obj = 3)
        self._splaytree.insert(key = 4, obj = 4)
        self._splaytree.insert(key = 6, obj = 6)

    def test_tree_structure_after_each_find_with_inorder_traversal(self):
        expected_inorder_list = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
        inorder_from_tree = []
        for i in range(1, 11): # 1 to 10
            self.assertEqual(self._splaytree.find(i), i, 'Element not found in Splay Tree!')
            for key, element in self._splaytree.traversal(want_in_order = True):
                inorder_from_tree.append(element)
            self.assertEqual(inorder_from_tree, expected_inorder_list, 'Tree ruined during insert splays')
            inorder_from_tree = []
        
    def tearDown(self):
        self._splaytree = None

#Allows running as python run.
if __name__ == '__main__':
    print('Splaytree tests')
    unittest.main()


'''
Created on Oct 29, 2015
@author: hari
'''
from __future__ import print_function
import unittest
from pycoils.stackNqueues.priorityqueue import PriorityQueueEntry

class PriorityQ_Element_Test_Init(unittest.TestCase):
    def setUp(self):
        unittest.TestCase.setUp(self)
        
    def test_priority_q_init(self):
        with self.assertRaises(ValueError):
            PriorityQueueEntry(item = None, priority = None)
        
    def tearDown(self):
        unittest.TestCase.tearDown(self)

class PriorityQ_Element_Test_Properties_With_None(unittest.TestCase):
    def setUp(self):
        pass 
        
    def test_priority_q_properties_with_none(self):
        with self.assertRaises(ValueError):
            PriorityQueueEntry(item = None, priority = 'Some Priority')
        with self.assertRaises(ValueError):
            PriorityQueueEntry(item = 'Some Item', priority = None)
        
    def tearDown(self):
        self._priority_q_entry = None

class PriorityQ_Element_Test_Properties_With_Known_Item_And_Priority(unittest.TestCase):
    def setUp(self):
        self._item = 'Some Item'
        self._priority = 100
        self._priority_q_entry = PriorityQueueEntry(item = self._item, priority = self._priority) 
        
    def test_priority_q_properties_with_known_properties(self):
        self.assertEqual(self._priority_q_entry.priority, self._priority, 'PriorityQ priority property did not matche expected')
        self.assertEqual(self._priority_q_entry.item, self._item, 'PriorityQ item property did not matche expected')
        
    def tearDown(self):
        self._priority_q_entry = None

class PriorityQ_Element_Test_Equality_of_Two_Entries(unittest.TestCase):
    def setUp(self):
        self._item_1 = 'Some Item'
        self._priority_1 = 100
        
        self._item_2 = 'Some Other Item with same priority'
        self._priority_2 = 100
        
        self._priority_q_entry_1 = PriorityQueueEntry(item = self._item_1, priority = self._priority_1) 
        self._priority_q_entry_2 = PriorityQueueEntry(item = self._item_2, priority = self._priority_2)
        
    def test_priority_q__test_equality_of_entries(self):
        self.assertEqual(self._priority_q_entry_1 == self._priority_q_entry_2, True, 'Priority Q entries with same priority did not pass equality')
        
    def tearDown(self):
        self._priority_q_entry = None


class PriorityQ_Element_Test_Greater_Than_And_Less_Than_Compare_of_Two_Entries(unittest.TestCase):
    def setUp(self):
        self._item_1 = 'Some Item'
        self._priority_1 = 100
        
        self._item_2 = 'Some Other Item with same priority'
        self._priority_2 = 101
        
        self._priority_q_entry_1 = PriorityQueueEntry(item = self._item_1, priority = self._priority_1) 
        self._priority_q_entry_2 = PriorityQueueEntry(item = self._item_2, priority = self._priority_2)
            
    def test_priority_q_test_greater_than_of_entries(self):
        self.assertEqual(self._priority_q_entry_2 > self._priority_q_entry_1, True, 'Priority Q entries did not pass greater than comparison')
    
    def test_priority_q_test_less_than_of_entries(self):
        self.assertEqual(self._priority_q_entry_1 < self._priority_q_entry_2, True, 'Priority Q entries did not pass less than comparison')
        
    def tearDown(self):
        self._priority_q_entry = None
        
if __name__ == '__main__':
    print('Priority Q entry tests')
    unittest.main()


import setuptools
import pycoils

with open("../README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pycoils",
    version=pycoils.version,
    author="Harisankar Krishna Swamy",
    license='Apache2',
    author_email="harisankar.krishna@outlook.com",
    description="A Python datastructure library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://bitbucket.org/harisankar-krishna-swamy-code/coils",
    packages=setuptools.find_packages(),
    python_requires='>=2.7, <4',
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ),
)


'''
Created on Aug 14, 2015
@author: hari
'''
from builtins import object
class Queue(object):
    """
    Implementation of queue in Python using list.
    """
    def __init__(self):
        self._list = []
    
    def enqueue(self, item):
        self._list.append(item)
    
    def dequeue(self):
        if len(self._list) == 0:
            return None
            
        item = self._list[0]
        del self._list[0]
        return item
    
    def __len__(self):
        return len(self._list)
    
    @property
    def size(self):
        return len(self._list)
    
    def __iter__(self):
        while len(self._list) != 0:
            yield self.dequeue()

    @property
    def front(self):
        if len(self._list) == 0:
            return None
        return self._list[0]
    
    def __str__(self):
        return '%s size %s Q %s' % (self.__class__.__name__, len(self), self._list) 


'''
Created on Oct 19, 2015
@author: hari
'''
from pycoils.trees.bst import BinarySearchTree

class SplayTree(BinarySearchTree):
    '''
    Splay tree is a binary search tree with splay operations to keep it balanced. Balanced search tree!
    '''
    def __init__(self, root_node=None):
        super(SplayTree, self).__init__(root_node = root_node)
    
    def _find_node_with_key_for_splay(self, key):
        '''
        Returns the node with key and its parent node too as (node, parent, grandparent). Splay tree rotation is acheved by knowing 
        parent and grand parent of a node. So when searching for  
        '''
        if self._root_node == None or key == None:
            return (None, None, None)
        current_node = self._root_node
        parent_node = None
        grand_parent_node = None
        
        while (current_node != None):
            if current_node.key == key:
                return (current_node, parent_node, grand_parent_node)
            grand_parent_node = parent_node
            parent_node = current_node
            if key > current_node.key:
                current_node = current_node.right_child
            else:    
                current_node = current_node.left_child
        #while
        #failed to find node but we fell off the tree at some point
        return (None, parent_node, grand_parent_node)
    
    def _rotate_right_for(self, aNode):
        aNode, parent, grandparent = self._find_node_with_key_for_splay(aNode.key)
        
        if grandparent != None:
            if parent == grandparent.left_child:
                grandparent.left_child = aNode
            else:
                grandparent.right_child = aNode
        else:
            self._root_node = aNode
        
        right_of_aNode = aNode.right_child
        parent.left_child = right_of_aNode
        aNode.right_child = parent
    
    def _rotate_left_for(self, aNode):
        aNode, parent, grandparent = self._find_node_with_key_for_splay(aNode.key)
        
        if grandparent != None:
            if parent == grandparent.left_child:
                grandparent.left_child = aNode
            else:
                grandparent.right_child = aNode
        else:
            self._root_node = aNode
        
        left_of_aNode = aNode.left_child
        parent.right_child = left_of_aNode
        aNode.left_child = parent
        
    def _splay_node_with_key(self, key):
        '''
        Splay operation
        '''
        node_to_splay = None
        parent = None 
        grand_parent = None
        
        while True:
            #this find operation needs to be done everytime if we do not store extra meta (isroot leftchild etc) on each node.
            #Assuming a billion nodes and the tree is almost balance which is the case, log2 (1 billion) < 25. Compared to that
            #storing meta for 1 billion nodes is too much space? 
            node_to_splay, parent, grand_parent = self._find_node_with_key_for_splay(key)
            if parent == None or (self._root_node.left_child == node_to_splay or self._root_node.right_child == node_to_splay):
                break#if node is root or child of root break
               
            if grand_parent.right_child != None and node_to_splay == grand_parent.right_child.left_child:
                self._rotate_right_for(node_to_splay)
                self._rotate_left_for(node_to_splay)
                continue
            if grand_parent.left_child != None and node_to_splay == grand_parent.left_child.right_child:
                self._rotate_left_for(node_to_splay)
                self._rotate_right_for(node_to_splay)
                continue
            if grand_parent.left_child != None and node_to_splay == grand_parent.left_child.left_child:
                self._rotate_right_for(parent)
                self._rotate_right_for(node_to_splay)
                continue
            if grand_parent.right_child != None and node_to_splay == grand_parent.right_child.right_child:
                self._rotate_left_for(parent)
                self._rotate_left_for(node_to_splay)
        #while
        #node_to_splay is either root or a child of root
        if parent == None:
            return
        if node_to_splay == parent.left_child:
            self._rotate_right_for(node_to_splay)
        else:
            self._rotate_left_for(node_to_splay)
        
    def remove(self, key):
        node_to_remove, parent, grand_parent = self._find_node_with_key_for_splay(key)# need to know to splay
        # remove as in BST
        BinarySearchTree.remove(self, key = key)
        #splay the parent from which we fell off the tree
        if parent != None:
            self._splay_node_with_key(parent.key)
        
    def find(self, key):
        node_with_key, parent, grand_parent = self._find_node_with_key_for_splay(key)# need to know to splay
        value = None
        if node_with_key != None:
            value = node_with_key.value
            self._splay_node_with_key(key)
        else:#node was not found so we splay the parent (node from which we fell off the tree)
            if parent != None:
                self._splay_node_with_key(parent.key)
        return value
    
    def insert(self, key=None, obj=None):
        #add the node.
        BinarySearchTree.insert(self, key=key, obj=obj)
        #then splay it to root
        if key != None and obj != None:
            self._splay_node_with_key(key)


'''
Created on Oct 21, 2015
@author: hari
'''
from builtins import str
from builtins import object
from functools import total_ordering

@total_ordering
class KeyValuePair(object):# Save space in key value pair by using slots for all client programs using this?! 
    __slots__ = 'key', 'value'# python created descriptors for the slots

    def __init__(self, key = None, value = None):
        self.key = key
        self.value = value
            
    def __eq__(self, other_kv_pair):
        if other_kv_pair is None:
            return False
        
        return self.key == other_kv_pair.key 
    
    def __lt__(self, other_kv_pair):
        if other_kv_pair is None:
            return False
        return self.key < other_kv_pair.key
    
    def __str__(self):
        return '%s key[%s] value[%s]' % (self.__class__.__name__,
                                         str(self.key), str(self.value))


'''
Created on Nov 8, 2015
@author: hari
'''
from __future__ import print_function
from builtins import range
import unittest
import gc
from pycoils.misc.internstore import InternStore
from tests.data.testdata import WrappedString

class Internstore_Test_With_None(unittest.TestCase):
    def setUp(self):
        self._internstore = InternStore()
    
    def test_intern_store_with_none(self):
        
        obj_to_intern = None
        id_of_obj_to_intern = id(obj_to_intern)
        interned_obj = self._internstore.intern(obj = obj_to_intern)
        
        id_of_interned_obj = id(interned_obj)    
        self.assertEqual(id_of_obj_to_intern, id_of_interned_obj, 'Interned object is not same as existing object id')
        
    def tearDown(self):
        pass


class Internstore_Test_With_1_Item_No_GC(unittest.TestCase):
    def setUp(self):
        self._internstore = InternStore()
    
    def test_intern_store_1_item_no_gc(self):
        
        obj_to_intern = WrappedString(string = 'Please intern me')
        id_of_obj_to_intern = id(obj_to_intern)
        interned_obj = self._internstore.intern(obj = obj_to_intern)
        
        id_of_interned_obj = id(interned_obj)    
        self.assertEqual(id_of_obj_to_intern, id_of_interned_obj, 'Interned object is not same as existing object id')
        
    def tearDown(self):
        pass

class Internstore_Test_With_2_Equal_Objects(unittest.TestCase):
    def setUp(self):
        self._internstore = InternStore()
    
    def test_intern_store_2_equal_objects(self):
        
        obj1_to_intern = WrappedString(string = 'Please intern me')
        id_of_obj1_to_intern = id(obj1_to_intern)
        interned_obj1 = self._internstore.intern(obj = obj1_to_intern)
        id_of_interned_obj1 = id(interned_obj1)
        self.assertEqual(id_of_obj1_to_intern, id_of_interned_obj1, 'Interned object is not same as existing object id')
        
        #new interning on a new object which is equal to the one in store.
        obj2_to_intern = WrappedString(string = 'Please intern me')# new object which is equal to the previous one
        id_of_obj2_to_intern = id(obj2_to_intern)
        interned_obj2 = self._internstore.intern(obj = obj2_to_intern)
        id_of_interned_obj2 = id(interned_obj2)    
        self.assertEqual(id_of_obj1_to_intern, id_of_interned_obj2, 'Interned object is not same as existing object id')
        #reference to the new object created will be lost as there is already a reference in the
        #store point to the same/equal object  
        self.assertNotEqual(id_of_obj2_to_intern, id_of_interned_obj2, 'Copy of an interned object was returned instead of original value')
    
    def tearDown(self):
        pass

class Internstore_Test_With_Many_Equal_Objects(unittest.TestCase):
    def setUp(self):
        self._internstore = InternStore()
    
    def test_intern_store_many_equal_objects(self):
        
        original_obj_to_intern = WrappedString(string = 'Please intern me')
        id_of_original_obj_to_intern = id(original_obj_to_intern)
        interned_obj = self._internstore.intern(obj = original_obj_to_intern)
        id_of_interned_obj = id(interned_obj)
        self.assertEqual(id_of_original_obj_to_intern, id_of_interned_obj, 'Interned object is not same as existing object id')
        
        count = 100000
        for i in range (0, count):
            new_obj = WrappedString(string = 'Please intern me')# new object which is equal to the previous one
            id_of_new_obj = id(new_obj)
            # sending this new object for interning will check the store and return an equal object
            # if it exists.
            interned_new_obj = self._internstore.intern(obj = new_obj)
            id_of_interned_new_obj = id(interned_new_obj)    
            self.assertEqual(id_of_original_obj_to_intern, id_of_interned_new_obj, 'Interned object is not same as existing object id')
        
    def tearDown(self):
        pass


if __name__ == '__main__':
    print('Interstore tests')
    unittest.main()


'''
Created on Oct 25, 2015
@author: hari
'''
from __future__ import print_function
from builtins import str
from pycoils.trees.bst import BinarySearchTree

def print_tree_inorder_using_stack(tree):
    traversed  = []
    for key, value in tree.inorder_traversal_with_stack():
        traversed.append((key, value))
    print('Tree stack based preorder traversal (key, value) pairs:')
    print(traversed)


def print_tree_preorder(tree):
    traversed  = []
    for key, value in tree.traversal(want_pre_order = True):
        traversed.append((key, value))
    print('Tree preorder traversal (key, value) pairs:')
    print(traversed)

def print_tree_postorder(tree):
    traversed  = []
    for key, value in tree.traversal(want_post_order = True):
        traversed.append((key, value))
    print('Tree postorder traversal (key, value) pairs:')
    print(traversed)

def print_tree_inorder(tree):
    traversed  = []
    for key, value in tree.traversal(want_in_order = True):
        traversed.append((key, value))
    print('Tree inorder traversal (key, value) pairs:')
    print(traversed)
    
if __name__ == '__main__':
    bst = BinarySearchTree()
    print('Node count is %s' % str(bst.node_count))
    print('Adding key value pairs (1, 1), (2, 2), .... (6, 6)')
    kvpairs = [(5, 5), (2, 2), (7, 7), (1, 1), (3, 3), (9, 9), (8, 8), (4, 4), (6, 6)]
    
    for kvpair in kvpairs:
        bst.insert(key = kvpair[0], obj = kvpair[1])
    
    print_tree_inorder(bst)
    print_tree_preorder(bst)
    print_tree_postorder(bst)
    
    #remove
    element_to_remove = 9
    print('removing element %s' % str(element_to_remove)) 
    bst.remove(key = element_to_remove)
    print_tree_inorder_using_stack(bst)
    
    #replace obj for a key
    key_to_replace = 1
    new_object_for_Key = 111
    
    bst.replace(key = key_to_replace, obj = new_object_for_Key)
    print_tree_inorder_using_stack(bst)
    
    
    